import numpy as np
from scipy.signal import butter, lfilter, resample, hilbert, filtfilt
import scipy
from sklearn import preprocessing


def highpass_filter(signal, hpf, fs):

    hp = hpf / (fs*0.5)
    hpb, hpa = butter(5, hp, 'highpass')
    hp_signal = filtfilt(hpb, hpa, signal, axis=0)

    return hp_signal

def lowpass_filter(signal, lpf, fs):

    lp = lpf / (fs*0.5)
    lpb, lpa = butter(5, lp, 'lowpass')
    lp_signal = filtfilt(lpb, lpa, signal, axis=0)

    return lp_signal

def downsample(data, rate, new_rate):
    num = int(len(data) * new_rate / rate)
    y = scipy.signal.resample(data, num)
    return y

# Wiener filter function
def wiener(sig, mysize=None, noise=None):
    sig = np.asarray(sig)

    if mysize is None:
        mysize = [3] * len(sig)
    mysize = np.asarray(mysize)
    if mysize.shape == ():
        mysize = np.repeat(mysize.item(), sig.ndim)

    # Estimate the local mean
    lMean = scipy.signal.correlate(sig, np.ones(mysize), 'same') / np.product(mysize, axis=0)

    # Estimate the local variance
    lVar = (scipy.signal.correlate(sig ** 2, np.ones(mysize), 'same') / np.product(mysize, axis=0) - lMean ** 2)

    # Estimate the noise power if needed.
    if noise is None:
        noise = np.mean(np.ravel(lVar), axis=0)

    res = sig - lMean
    res *= (1 - noise / lVar)
    res += lMean
    out = np.where(lVar < noise, lMean, res)

    return out

def local_wiener(sig, mysize=21, local_size=10, fs=2000):
    sig = np.asarray(sig)

    pn = np.var(np.reshape(sig, (-1, local_size)), axis=-1)
    pn_med = np.quantile(pn, 0.25)
    pn_min = np.min(pn)
    pn_m = (pn_med + pn_min) / 2

    sig_filted = wiener(sig, mysize, noise=pn_m)

    return sig_filted

def hs_process4seg(data, sr):
    data_w = local_wiener(data, mysize=11, local_size=50, fs=sr)
    data_w = downsample(data_w, sr, 800)
    data_w -= np.mean(data_w)

    data_h = data_w - lowpass_filter(data_w, 15, sr)
    data_h = data_h - highpass_filter(data_h, 300, sr)
    data_h = preprocessing.minmax_scale(data_h)

    data_h = np.expand_dims(data_h, -1)

    return data_h

def hs_process_batch4seg(data, sr):
    hss = []
    for single_hs in data:
        data_w = local_wiener(single_hs, mysize=11, local_size=50, fs=sr)
        data_w = downsample(data_w, sr, 800)
        data_w -= np.mean(data_w)

        data_h = data_w - lowpass_filter(data_w, 15, sr)
        data_h = data_h - highpass_filter(data_h, 300, sr)
        data_h = preprocessing.minmax_scale(data_h)

        data_h = np.expand_dims(data_h, -1)
        hss.append(data_h)

    hss = np.array(hss)

    return hss

def hs_process4murmur(data, sr):
    data_w = local_wiener(data, mysize=11, local_size=50, fs=sr)
    data_w = downsample(data_w, sr, 1000)
    data_w -= np.mean(data_w)

    data_h = data_w - lowpass_filter(data_w, 15, sr)
    # data_h = data_h - highpass_filter(data_h, 500, sr)
    data_h = preprocessing.scale(data_h)

    data_h = np.expand_dims(data_h, -1)

    return data_h