import numpy as np
import random
import os
import re


def convert_to_one_hot(Y, C):
    Y = np.eye(C)[Y]
    return Y


class HSDataset():
    def __init__(self, batch_size, path, normal_list, murmur_list, unknown_list):
        self.batch_size = batch_size
        self.path = path
        self.normal_list = normal_list
        self.murmur_list = murmur_list
        self.unknown_list = unknown_list

    def inputs(self, is_training=True):
        normal_set = open(self.normal_list, 'r').read().splitlines()
        murmur_set = open(self.murmur_list, 'r').read().splitlines()
        unknown_set = open(self.unknown_list, 'r').read().splitlines()

        train_data = []
        test_data = []
        train_labels = []
        test_labels = []
        if is_training:
            normal_data_list = random.sample(normal_set, self.batch_size)
            murmur_data_list = random.sample(murmur_set, self.batch_size)
            unknown_data_list = random.sample(unknown_set, self.batch_size)

            for i, data_index in enumerate(normal_data_list):
                normal_path = os.path.join(self.path, 'normal_data')
                data_path = os.path.join(normal_path, data_index)
                ref = 0

                data = np.load(data_path)
                label = convert_to_one_hot(ref, 3)

                train_data.append(data)
                train_labels.append(label)
            
            for i, data_index in enumerate(murmur_data_list):
                murmur_path = os.path.join(self.path, 'murmur_data')
                data_path = os.path.join(murmur_path, data_index)
                ref = 1

                data = np.load(data_path)
                label = convert_to_one_hot(ref, 3)

                train_data.append(data)
                train_labels.append(label)
            
            for i, data_index in enumerate(unknown_data_list):
                unknown_path = os.path.join(self.path, 'unknown_data')

                data_path = os.path.join(unknown_path, data_index)
                ref = 2

                data = np.load(data_path)
                label = convert_to_one_hot(ref, 3)

                train_data.append(data)
                train_labels.append(label)

            train_data = np.array(train_data)
            train_labels = np.asarray(train_labels)

            return train_data, train_labels

if __name__ == '__main__': 
    PATH = '/data/heart_sound/physionet2022/murmur_training_set/'
    NORMAL_INDEX = '/data/heart_sound/physionet2022/murmur_training_set/NORMAL_RECORDS'
    MURMUR_INDEX = '/data/heart_sound/physionet2022/murmur_training_set/MURMUR_RECORDS'
    UNKNOWN_INDEX = '/data/heart_sound/physionet2022/murmur_training_set/UNKNOWN_RECORDS'

    # Train
    dataset = HSDataset(batch_size=10, path=PATH, normal_list=NORMAL_INDEX, murmur_list=MURMUR_INDEX, unknown_list=UNKNOWN_INDEX)
    x, y = dataset.inputs(is_training=True)
    print(x.shape)
    print(y.shape)
    
